# Copyright 2014-2015 Jan Decaluwe
#
# This file is part of Urubu.
#
# Urubu is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Urubu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Urubu.  If not, see <http://www.gnu.org/licenses/>.

# Python 3 idioms
from __future__ import unicode_literals
from io import open

import posixpath

import markdown
import logging
logging.captureWarnings(False)

from markdown import Extension
from markdown.extensions import toc 
from markdown.treeprocessors import Treeprocessor
from markdown.inlinepatterns import ReferencePattern, REFERENCE_RE, SHORT_REF_RE
from markdown.inlinepatterns import SimpleTagPattern, SMART_EMPHASIS_RE

from urubu import UrubuWarning, urubu_warn, UrubuError, _warning, _error


def _set_dl_class(tree):
    for item in tree:
        if item.tag == 'dl':
            item.set('class', 'dl-horizontal')
        _set_dl_class(item)


class DLClass(Treeprocessor):

    def run(self, root):
        _set_dl_class(root)
        return None


class DLClassExtension(Extension):

    """Add 'dl-horizontal' class to definition list elements (for bootstrap)."""

    def extendMarkdown(self, md, md_globals):
        md.treeprocessors.add('dlclass', DLClass(md), "_end")


def _set_table_class(tree):
    for item in tree:
        if item.tag == 'table':
            item.set('class', 'table')
        _set_table_class(item)


class TableClass(Treeprocessor):

    def run(self, root):
        _set_table_class(root)
        return None


class TableClassExtension(Extension):

    """Add 'table' class to table elements (for bootstrap)."""

    def extendMarkdown(self, md, md_globals):
        md.treeprocessors.add('tableclass', TableClass(md), "_end")


class ProjectReferencePattern(ReferencePattern):

    def handleMatch(self, m):
        try:
            ref = m.group(9)
        except IndexError:
            ref = None
        shortref = False
        if not ref:
            # if we got something like "[Google][]" or "[Google]"
            # we'll use "google" as the id
            ref = m.group(2)
            shortref = True

        # Clean up linebreaks in ref
        ref = self.NEWLINE_CLEANUP_RE.sub(' ', ref)

        text = m.group(2)
        id = ref.lower()

        if id in self.markdown.references:
            href, title = self.markdown.references[id]
        else:
            anchor = None
            if '#' in ref:
                ref, anchor = ref.split('#', 1)
            this = self.markdown.this
            if not posixpath.isabs(ref):
                # treat empty ref as reference to current page
                if not ref: 
                    ref = this['components'][-1]
                rootrelpath = '/' + '/'.join(this['components'][:-1])
                id = posixpath.normpath(posixpath.join(rootrelpath, ref))
                id = id.lower()
            else:
                id = ref.lower()
            ref = ref.lower()
            if ref in self.markdown.site['reflinks']:
                if (ref != id) and (id in self.markdown.site['reflinks']):
                    raise UrubuError(_error.ambig_ref_md, msg=ref, fn=this['fn'])
                id = ref
            if id in self.markdown.site['reflinks']:
                item = self.markdown.site['reflinks'][id]
                href, title = item['url'], item['title']
                if shortref:
                    text = title
                    if anchor is not None:
                        text = anchor
                if anchor is not None:
                    anchor = toc.slugify(anchor, '-')
                    href = '%s#%s' % (href, anchor)
                    anchorref = '%s#%s' % (id, anchor)
                    self.markdown.this['_anchorrefs'].add(anchorref)
            else:  # ignore undefined refs
                urubu_warn(_warning.undef_ref_md, msg=ref, fn=this['fn'])
                return None

        return self.makeTag(href, title, text)


class ProjectReferenceExtension(Extension):

    """Overwrite reference pattern with project reference extension."""

    def extendMarkdown(self, md, md_globals):
        md.inlinePatterns['reference'] = ProjectReferencePattern(
            REFERENCE_RE, md)
        md.inlinePatterns['short_reference'] = ProjectReferencePattern(
            SHORT_REF_RE, md)


class ExtractAnchorsClass(Treeprocessor):

    def run(self, tree):
        this = self.markdown.this
        thisid = this['id']
        components = this['components']
        for item in tree:
            if 'id' in item.attrib:
                self.markdown.anchors.add("%s#%s" % (thisid, item.attrib['id']))
                # add special version for index files
                if components[-1] == 'index':
                    navid = thisid[:-6] # remove trailing backslash also
                    self.markdown.anchors.add("%s#%s" % (navid, item.attrib['id']))
        return None


class ExtractAnchorsExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.treeprocessors.add('extractanchors', ExtractAnchorsClass(md), "_end")


# extension for the <mark> tag
class MarkTagExtension(Extension):
    def extendMarkdown(self, md, md_globals):
        # emphasis2 is the one with underscores, use the smart version
        md.inlinePatterns['emphasis2'] = SimpleTagPattern(SMART_EMPHASIS_RE, 'mark')






# -*- coding: utf-8 -*-
"""
    jinja2._compat
    ~~~~~~~~~~~~~~

    Some py2/py3 compatibility support based on a stripped down
    version of six so we don't have to depend on a specific version
    of it.

    :copyright: Copyright 2013 by the Jinja team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""
import sys

PY2 = sys.version_info[0] == 2
PYPY = hasattr(sys, 'pypy_translation_info')
_identity = lambda x: x


if not PY2:
    unichr = chr
    range_type = range
    text_type = str
    string_types = (str,)
    integer_types = (int,)

    iterkeys = lambda d: iter(d.keys())
    itervalues = lambda d: iter(d.values())
    iteritems = lambda d: iter(d.items())

    ifilter = filter
    imap = map
    izip = zip
    intern = sys.intern

    import socketserver
    import http.server as httpserver

else:
    unichr = unichr
    text_type = unicode
    range_type = xrange
    string_types = (str, unicode)
    integer_types = (int, long)

    iterkeys = lambda d: d.iterkeys()
    itervalues = lambda d: d.itervalues()
    iteritems = lambda d: d.iteritems()

    from itertools import imap, izip, ifilter
    intern = intern

    import SocketServer as socketserver
    import SimpleHTTPServer as httpserver




# Copyright 2014 Jan Decaluwe
#
# This file is part of Urubu.
#
# Urubu is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Urubu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Urubu.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

import argparse

import os

from urubu import __version__
from urubu import project

from urubu._compat import socketserver, httpserver
from urubu.httphandler import AliasingHTTPRequestHandler

def serve(baseurl):
    """HTTP server straight from the docs."""
    # allow running this from the top level
    if os.path.isdir('_build'):
        os.chdir('_build')
    # local use, address reuse should be OK
    socketserver.TCPServer.allow_reuse_address = True
    PORT = 8000
    handler = AliasingHTTPRequestHandler
    httpd = socketserver.TCPServer(('', PORT), handler)
    httpd.baseurl = baseurl
        
    print("Serving at port {}".format(PORT))
    if httpd.baseurl: print("Using baseurl {}".format(httpd.baseurl))
    httpd.serve_forever()

def main():
    parser = argparse.ArgumentParser(prog='python -m urubu')
    parser.add_argument('--version', action='version', version=__version__)
    parser.add_argument('command', choices=['build', 'serve'])
    args = parser.parse_args()
    if args.command == 'build':
        project.build()
    elif args.command == 'serve':
        proj = project.load()
        serve(proj.site['baseurl'])



# Copyright 2014 Jan Decaluwe
#
# This file is part of Urubu.
#
# Urubu is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Urubu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Urubu.  If not, see <http://www.gnu.org/licenses/>.

__version__ = "1.2.0"

from warnings import warn

class UrubuWarning(UserWarning):
    pass

def urubu_warn(kind, msg='', fn=''):
    if fn:
        fn = 'in ' + fn + ': '
    if msg:
        msg = ': ' + "'{}'".format(msg)
    warn(fn + kind + msg, UrubuWarning, stacklevel=2)


class UrubuError(Exception):
    def __init__(self, kind, msg='', fn=''):
        self.kind = kind
        self.msg = msg
        self.fn = fn 
    def __str__(self):
        fn = self.fn 
        if fn:
            fn = 'in ' + fn + ': '
        msg = self.msg
        if msg:
            msg = ': ' + "'{}'".format(msg)
        return fn + self.kind + msg

class _error():
    pass

class _warning():
    pass

_warning.no_yamlfm = "No yaml front matter - ignored"
_warning.undef_tag_layout = "Tags defined, but no tag layout found"
_warning.undef_ref_md = 'Undefined reference'
_warning.undef_anchor = "Undefined anchor"

_error.ambig_refid = "Ambiguous reference id"
_error.undef_ref = "Undefined reference"
_error.ambig_ref = "Ambiguous reference, cannot resolve"
_error.undef_info = "Missing attribute"
_error.date_format = "Date format error - should be YYYY-MM-DD"
_error.undef_key = "Undefined key"
_error.undef_content = "No 'content' or 'order' specified"
_error.undef_reflink_key = "Undefined key in site reflink"
_error.ambig_ref_md = 'Ambiguous reference' 
_error.no_index = 'Missing index file'
